#include <Cdef21369.h>
#include <def21369.h>
#include <21369.h>

#include "Flash_Store_Process.h"
#include "param_mem.h"

 // EXTERNALLY DEFINED VARIABLES ======================================================
 extern struct dm_int_params iparams[NINTPARAMS];
 extern struct dm_flt_params fparams[NFLTPARAMS];
 
 // GLOBAL VARIABLES ==================================================================
 
/*============ FLASH Memory Process Control ======== */
/* This structure of variables are used with the flash
 * erase and buffer programming of the flash.  These
 * processes take a lot of time. The control variables
 * store the state of the process so we can do the full
 * erasing and writing and programming of 8 words at a time
 * in pieces when processing time is available.  We can 
 * make the Flash storage of our parameters and curves a
 * lowest priority sort of task.
 *
 * FLASH STATUS WORD: The flash_status, status word has the 
 * following definition.
 *			
 *		0x00000000 = Flash Process is Inactive
 *		0x00000001 = Flash Erase if Active
 *		0x00000002 = Flash Ready for Next Buffer 
 *      0x00000004 = Flash Buffer Programming is Active
 *		0x00000008 = 
 *
 */

	struct flash_control
	{
		int	flash_status; 			/* see above for status definitions */
		int	prm_store_ndx;			/* parameter index (i) used in writing buffer */
		int prm_store_cnt;			/* parameter count (k) used for dimensions > 1 */
		int buf_store_ndx;			/* Buffer index (m) used when writing Flash Buffer */
		int *flash_polling_addr;	/* Address to use for status polling of the Flash */
		int sector_ndx;				/* Flash sector number where we want to store parameters */
		int base_addr;
		int sect_size;
	};

	struct flash_control fc = {0, 0, 0, 0, (int *) 0, 20, 0, 0}; // note default sector index is 20
 //====================================================================================

int Init_Flash_Process(int flash_base_address, int flash_sector_size)
{
	fc.base_addr = flash_base_address;
	fc.sect_size = flash_sector_size;
	return 0;
}

//================================================================
// This routine is a state machine of sorts to keep track of where 
// we are in the erasing and programming of the Flash. This allows 
// the main program to make small steps in the Flash store process 
// and work them in as low priority tasks.  After all, the Flash 
// store process takes a lot of time processing within the Flash 
// memory chip, and we are not anxious to write the Flash very many 
// times in its life cycle.

int Flash_Store_Process(int *req_store)
{
	int DQ7=0;
	int DQ6_A=0;
	int DQ6_B=0; 
	int test_cnt=0;
	
	if (fc.flash_status != 0) 
	{ 
		// Flash Erase Active -------------------------------------------------------
		if      (fc.flash_status == 1) 
		{
			DQ7 = *fc.flash_polling_addr;
			DQ7 &= 0x00000080; // get bit 7
			if (DQ7 > 0) fc.flash_status = 2; 
								//status = 2 is Flash ready for programming	
		}
		// Flash Ready for Programming ----------------------------------------------
		else if (fc.flash_status == 2) 
		{
			if      (fc.prm_store_ndx < NINTPARAMS) 
			{ store_iparams(); fc.flash_status = 4;} // flash buffer programming active
			
			else if (fc.prm_store_ndx < (NINTPARAMS + NFLTPARAMS)) 
			{ store_fparams(); fc.flash_status = 4; } // flash buffer programming active
			
			else     
			{ fc.flash_status = 0; } // all parameters have been stored
									 // Flash Process is Inactive			
		}
		// Flash Buffer Programming Active ------------------------------------------
		else if (fc.flash_status == 4) 
		{
			DQ6_A  = *fc.flash_polling_addr;
			DQ6_B  = *fc.flash_polling_addr;
		  	DQ6_A &= 0x00000040; // get bit 6
		  	DQ6_B &= 0x00000040; // get bit 6
		
		  	if (test_cnt++ > 20000)
		  	{ 
		  		DQ6_A = DQ6_B;
				test_cnt = 0;
		  			sprintf(msg_str,"TIMED OUT *****");message2(msg_str);
		  	}
	  	
		  	if (DQ6_A == DQ6_B)  // If programming algorithm is still running - bit will toggle
			{
				fc.flash_status = 2; // status = 2 is Flash ready for buffer programming	
			}
		}
	}
	
	else if(*req_store == 1) // Parameter Storage Requested ?
	{
		fc.prm_store_ndx = 0; // initialize param index (i)
		fc.prm_store_cnt = 0; // initialize param count (k)
		fc.buf_store_ndx = 0; // initialize buffer index (m)
		
		flash_sector_erase(fc.sector_ndx);
		fc.flash_status = 1;  // Flash Erase Active
		
		*req_store = 0; // reset the request		
				test_cnt = 0;
	}

	if (fc.flash_status > 0) return 1;
	else                     return 0;

} // Flash_Store_Process.end	


//=========================================================================================
// Store the Parameters in the Flash Memory

int store_iparams(void)

{
	int   	i=0,j=0,k=0;
	int		data_buffer[8];
	float 	fdata_buffer[8];
	
	i = fc.prm_store_ndx;
	k = fc.prm_store_cnt;
	
	while((j < 8) && (i < NINTPARAMS))
	{
		data_buffer[j] = *(iparams[i].sm_set + k);
		j++;
		k++;
		if(k >= iparams[i].dim) {k=0; i++;} //if done with param [i] go to next param [i+1]
	}
	
	flash_buffer_prog(fc.sector_ndx, fc.buf_store_ndx, (void *) data_buffer, j);
	
	fc.buf_store_ndx++;
	fc.prm_store_ndx = i;
	fc.prm_store_cnt = k;

	return 0;
}

int store_fparams(void)

{
	int   	i=0,j=0,k=0;
	float 	fdata_buffer[8];
	
	i = fc.prm_store_ndx - NINTPARAMS;
	k = fc.prm_store_cnt;
	
	while((j < 8) && (i < NFLTPARAMS))
	{
		fdata_buffer[j] = *(fparams[i].sm_set + k);
		j++;
		k++;
		if(k >= fparams[i].dim) {k=0; i++;} //if done with param [i] go to next param
	}
	
	flash_buffer_prog(fc.sector_ndx, fc.buf_store_ndx, (void *) fdata_buffer, j);
	
	fc.buf_store_ndx++;
	fc.prm_store_ndx = i + NINTPARAMS;
	fc.prm_store_cnt = k;

	return 0;
}

//=========================================================================================
// Erase a sector in the Flash Memory

int flash_sector_erase(int sector_ndx)
{
	int		*flash_cmd;
	
	/* Ensure sector is a parameter storage sector */
	if (sector_ndx < 20 || sector_ndx > 31) return -1;
	
	/* Unlock the Flash Memory (byte address used)*/
	 flash_cmd = (int *) (fc.base_addr + 0x00000AAA);
	*flash_cmd = 0x000000AA;
	
	 flash_cmd = (int *) (fc.base_addr + 0x00000555);
	*flash_cmd = 0x00000055;
	
	/* Write Setup Command  (byte address used)*/
	 flash_cmd = (int *) (fc.base_addr + 0x00000AAA);
	*flash_cmd = 0x00000080;
	
	/* Unlock the Flash Memory Again (byte address used)*/
	 flash_cmd = (int *) (fc.base_addr + 0x00000AAA);
	*flash_cmd = 0x000000AA;
	
	 flash_cmd = (int *) (fc.base_addr + 0x00000555);
	*flash_cmd = 0x00000055;
	
	/* Write Sector Erase Command */
	 flash_cmd = (int *) (fc.base_addr + sector_ndx * fc.sect_size);
	*flash_cmd = 0x00000030;
	
	fc.flash_polling_addr = flash_cmd;
		
	return 0;
}

//=========================================================================================
// RESET Flash 
int reset_flash(void)
{
	int  *flash_cmd;
	
	 flash_cmd = (int *) (fc.base_addr);
	*flash_cmd = 0x000000F0;
	return 0;	
}

//******************************************************************
// Programming the Flash through the Flash Write buffer
//
// This routine is written with a couple assumptions
//
//		1. The data bus input on the Flash is 8 Bits wide.

//		2. Packing has been disabled through the AMI connecting this
//			Flash memory.
//
//      3. The AMI has been setup according to 
//		    //Flash Settings
//		    usreg2 = WS23|AMIEN|BW8|PKDIS;
//		    *pAMICTL1= usreg2;

//******************************************************************
// This routine programs up to 32 bytes of address data pairs into a specific 
// Flash sector.  The 32 bytes corresponds to a maximum of 8 32 bit words.
//
// To program more than 8 words into the Flash sector you need to re-call 
// this routine after you have incremented the write buffer index (wr_buf_ndx)
// by 32 and incremented data_buf pointer by nwords.  The first wr_buf_ndx = 0.
//
int flash_buffer_prog(int sector_ndx, int wr_buf_ndx, void* data_buf, int nwords)
{
	int		*flash_cmd, *mem_ptr, *buf_ptr;
	int		i,n;
	
	buf_ptr = (int *) data_buf; // data_buf might be intergers or floats
	
	if(nwords > 8) n = 31; else n = 4*nwords - 1;
	
	/* Unlock the Flash Memory (byte address used)---------------------------*/
	 flash_cmd = (int *) (fc.base_addr + 0x00000AAA);
	*flash_cmd = 0x000000AA;
	
	 flash_cmd = (int *) (fc.base_addr + 0x00000555);
	*flash_cmd = 0x00000055;
	
	/* Write Buffer Load Command --------------------------------------------*/
	 flash_cmd = (int *) (fc.base_addr + sector_ndx * fc.sect_size);
	*flash_cmd = 0x00000025;
	
			fc.flash_polling_addr = flash_cmd; // Used for polling DQ6

	
	/* Write Word Count -----------------------------------------------------*/
	 //flash_cmd = "No Change"
	*flash_cmd = n;  // (N - 1) where N = bytes in buffer
	
	/* Write Words into Buffer ----------------------------------------------*/
	mem_ptr  = flash_cmd + (wr_buf_ndx*32);
	
	for (i=0;i<nwords;i++)
	{
		flash_buffer_write( buf_ptr, mem_ptr);
		buf_ptr++;
		mem_ptr += 4;	
	}
		
	/* Program the flash buffer into flash ----------------------------------*/
	 //flash_cmd = "No Change, still the sector address
	*flash_cmd = 0x00000029;
	
	return 0;
}


/*****************************************************************************/
// Write a 4 byte word into the Flash buffer 1 byte at a time.
// 
// The bytes are stored Little-Endian.  That is the LSB is stored 
// at the lower address and the MSB is stored at the higher.

int flash_buffer_write( void *word32, void *flash8)   
{
    int *memptr;
    int *datptr;
	int data32;
	
	datptr = (int *)word32; // convert void pointer
	memptr = (int *)flash8;	// convert void pointer
	
	data32 = *datptr; // get 4 byte data value
	
	*memptr = data32;
	
	memptr++;
	data32  = data32 >> 8;	
	*memptr = data32;
	
	memptr++;
	data32  = data32 >> 8;	
	*memptr = data32;
	
	memptr++;
	data32  = data32 >> 8;
	*memptr = data32;

    return 0;
}

/*****************************************************************************/
// The Flash sector offset must be added to the memory pointer before calling
// flash_read.

int flash_read_param(int *flash8, int *word32)
{
	flash_read((void *) (flash8 + fc.sector_ndx * fc.sect_size), (void *)  word32);
	return 0;	
}

/*****************************************************************************/
// Read a 4 byte word from the Flash 1 byte at a time and re-assemble the 
// 4 byte word.
// 
// The bytes are stored Little-Endian.  That is the LSB is stored 
// at the lower address and the MSB is stored at the higher.

int flash_read(void *flash8, void *word32)    
{
	int data32;
    int *memptr;
    int *ret_val;
	
	data32 = 0;
	memptr  = (int *)flash8;
	ret_val = (int *)word32;
	
	data32 = 0xFF & *memptr;
	memptr++;

	data32 = data32 | ((0xFF & *memptr) << 8);
	memptr++;
	
	data32 = data32 | ((0xFF & *memptr) << 16);
	memptr++;
	
	data32 = data32 | ((0xFF & *memptr) << 24);
		
	*ret_val = data32; // recall ret_val is the re-cast version of word32
	
    return 0;
}



