/* Sets up the SDRAM controller to access SDRAM. In this file are two subroutines, the first
   to set up the SHARC's PLL, and the second to set up the SDRAM controller.
   CLKIN= 31.24 MHz, Multiplier= 21, Divisor= 2, CCLK_SDCLK_RATIO 2.5
   Core clock = (31.24 MHz * 21) /2 = 328.02 MHz
*/

#include <Cdef21369.h>
#include <def21369.h>
#include <21369.h>
#include "cogging.h"
#include "mfctest_c.h"



void    initPLL_SDRAM(void)
{
    unsigned int i, usreg1, usreg2;


// CLKIN= 31.24 MHz, Multiplier= 21, Divisor = 2, CCLK_SDCLK_RATIO 2.5
// Core clock = (31.24 MHz * 21)/2 = 328.02 MHz
    usreg1 = PLLM21|DIVEN|PLLD2;

// CLKIN= 65 MHz, Multiplier= 10, Divisor= 2, CCLK_SDCLK_RATIO 2.5
// Core clock = (65 MHz * 10) /2 = 325 MHz
//    ustat3 = PLLM10|INDIV;


    // Set the Core clock (CCLK) to SDRAM clock (SDCLK) ratio to 2.5
	usreg1 &= ~(SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4);
	usreg1 |= SDCKR2_5;
    *pPMCTL = usreg1;
    usreg1 |= PLLBP;
	usreg1 &= ~(DIVEN);
    *pPMCTL = usreg1;

     // Wait for at least 4096 cycles for the pll to lock
    for (i=0; i<5000; i++)  asm("nop;");

    usreg1 = *pPMCTL;
    usreg1 &= ~(PLLBP);
    *pPMCTL = usreg1;

    // Programming SDRAM control registers.
    // RDIV = ((f SDCLK X t REF )/NRA) - (tRAS + tRP )

    // CCLK_SDCLK_RATIO==2.5
    usreg2 = 0x7F4; // (131*(10^6)*64*(10^-3)/4096) - (7+3) = 2427 (0x97B)


    //===================================================================
    //
    // Configure SDRAM Control Register (SDCTL) for the Micron MT48LC4M32
    //
    //  SDCL3  : SDRAM CAS Latency= 3 cycles
    //  DSDCLK1: Disable SDRAM Clock 1
    //  SDPSS  : Start SDRAM Power up Sequence
    //  SDCAW8 : SDRAM Bank Column Address Width= 8 bits
    //  SDRAW12: SDRAM Row Address Width= 12 bits
    //  SDTRAS7: SDRAM tRAS Specification. Active Command delay = 7 cycles
    //  SDTRP3 : SDRAM tRP Specification. Precharge delay = 3 cycles.
    //  SDTWR2 : SDRAM tWR Specification. tWR = 2 cycles.
    //  SDTRCD3: SDRAM tRCD Specification. tRCD = 3 cycles.
    //
    //--------------------------------------------------------------------

    
    usreg1 = SDCL3|DSDCLK1|SDPSS|SDCAW8|SDRAW12|SDTRAS7|SDTRP3|SDTWR2|SDTRCD3;

    *pSDCTL = usreg1;

    // Change this value to optimize the performance for quazi-sequential accesses (step > 1)
    #define SDMODIFY 1                    // Setting the Modify to 1
    usreg2 |= (SDMODIFY<<17)|SDROPT; 	  // Enabling SDRAM read optimization

    *pSDRRC = usreg2;

    // Note that MS2 & MS3 pin multiplexed with flag2 & flag3.
    usreg1 = *pSYSCTL;
    usreg1 |= MSEN;    //This setting allows SDRAM access
    *pSYSCTL = usreg1;

    //===================================================================
    // Mapping Bank 0 to SDRAM
    // Mapping Bank 1,2,3 to Asynchronous Memory    
     //===================================================================
   	usreg1  = *pEPCTL;
    usreg1 |= B0SD;
    usreg1 &= ~(B1SD|B2SD|B3SD);
    *pEPCTL = usreg1;

    //===================================================================
    // Configure AMI Control Register (AMICTL1) Bank 1 for the S29GL128N
    //
    //  WS23 : Wait States= 23 cycles
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //Flash Settings
    usreg2 = WS23|AMIEN|BW8|PKDIS;
    *pAMICTL1= usreg2;

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL2) Bank 2 for the MAX II Accesses
    //
    //  WS2 : Wait States= 23 cycles
    //  HC1  : Bus Hold Cycle (at end of write access)= 1 cycle.
    //  AMIEN: Enable AMI
    //  BW32  : External Data Bus Width= 32 bits.
    //	ACKEN: ACK Pin enable
    //--------------------------------------------------------------------

    //MAX II Settings
    usreg2 = WS31|HC1|IC5|AMIEN|BW32|ACKEN;
	//  usreg2 = WS5|HC2|IC7|AMIEN|BW32|ACKEN;
    *pAMICTL2= usreg2;

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL3) Bank 3 for the FPGA Accesses
    //
    //  WS2 : Wait States= 10 cycles
    //  HC1  : Bus Hold Cycle (at end of write access)= 2 cycle.
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 32 bits.
    //	ACKEN: ACK Pin enable
    //--------------------------------------------------------------------

    //FPGA Settings
    usreg2 = WS31|HC2|AMIEN|BW32|ACKEN;
    *pAMICTL3 = usreg2;

	}// initPLL_SDRAM.end      



